#include "ti82.h"
#include "keys.inc"

;-------------------------------------------------------------------------------
;Variables in text-memory:
;-------------------------------------------------------------------------------
ORIGLVL = TEXT_MEM+17   ;pointer to start of level
StringPlace = TEXT_MEM+22
ZeroByte = TEXT_MEM+27
;hiscoreTemp = TEXT_MEM+28
Count = TEXT_MEM+30
TempByte = TEXT_MEM+31
MASK = TEXT_MEM+32
ANISCROLLC = TEXT_MEM+33
KEYPRESS = TEXT_MEM+34
LvlNo           = TEXT_MEM+50
Top             = TEXT_MEM+51
LvlMinusTop     = TEXT_MEM+52

Levels          = APD_BUF


;-------------------------------------------------------------------------------
BITMAPMEM = APD_BUF ;this points to the graph mem

 .org START_ADDR
StartAddr =$-3
 .DB "Plain Jump v1.4",0

;-------------------------------------------------------------------------------
;Program start!
;-------------------------------------------------------------------------------
Start:

 res 1,(iy+13)


Title:
 set 7,(iy+$14)
 call clrtxtmem

 call _GRBUFCLR
 ld   de,GRAPH_MEM
 ld   bc,400
 ld   hl,TitlePicture
 ldir
 ld   hl,SamStr
 ld   de,50*256+32
 call DisplayText
 ld   de, 36*256+44
 call DisplayText
 ld   de, 43*256+30
 call DisplayText



 ld de,57*256+10
 call DisplayText

 ld de,57*256+44
 call DisplayText

 ld  de,57*256+65
 ld hl,hiscore
 call LD_HL_MHL
 call HL_DECIMAL_MENU

 call fastCopy

Wait:
 call OTH_PAUSE

 ld hl,LevelSaved
 ld a,(hl)
 and a
 jr nz,LoadGame
 call LoadLevel
 jr c,ExitGame 

 jp RegularGame

OTH_PAUSE:
 call GET_KEY
 cp G_CLEAR
 jr z,ExitGame
 cp G_ENTER
 jr nz,OTH_PAUSE
 ret

LSaveGame:
 ld  (BALLpos), bc
 ld  a, 1
 jr DoExit
ExitGame:
 xor a
DoExit: 
 ld  hl, LevelSaved
 ld (hl),a
 
 ;call   _GRBUFCLR
 ;call _clrScrnFull           
 set     1,(iy+13)       
 ret
clrtxtmem:
 ld bc,$7f
 ld hl, TEXT_MEM

ClearMem:
 ld (hl),0
 ld d,h
 ld e,l
 inc de
 ldir
 ret

LoadGame:
 ld (hl),0
 res 7,(iy+$14)
 ld hl,LEVELNAME
 call ReadLevel
 push hl
 ld a,(LEVELY)
 ld e,a
 ld d,0
 add hl,de
 ld bc,(BALLpos)
 pop de			; keep trace of level origin (or else you'll get bonus too easily)
 call Setup
 jr StartGameLoop

Setup:
	ld      (ORIGLVL), de
	ld      (LEVELPT), hl
	ld      HL, BlackBall1
	ld      (PLAYERPT1), HL
      ret

LostLife:
	xor     A
	ld      (SPEED), A
NoFade:
	ld      A, (LIFE)
	dec     A
	jp      z, GameOver
	ld      (LIFE), A
lost2:
	xor     A
	ld      HL, SCROLLY
	ld      (HL), A
	ld      HL, (LEVELPT)
FindStone:
	dec     HL
	ld      (LEVELPT), HL
	ld      A, (LEVELY)
	dec     A
	ld      (LEVELY), A
	jr      z, StartAfterLife
	ld      C, $80
	ld      B, 1
FindStone1:
	ld      A, (HL)
	and     C
	jr      nz, FoundDone
	srl     C
	jr      z, FindStone
	ld      A, B
	add     A, 12
	ld      B, A
	jr      FindStone1
FoundDone:
	ld      C, 46
      ld      a,b
      cp      6
      jr      c,StartGameLoop
      dec     b
      dec     b
      dec     b
      dec     b
	jr      StartGameLoop



RegularGame:
      push hl
      ld     hl,LEVELY
      ld     bc,17
      call ClearMem
      pop hl

      ld      a,6
	ld      (LIFE), A
	ld d,h
	ld e,l
SetParams:
      call Setup
StartAfterLife:
 ld   b, 40
 ld   c, 46
StartGameLoop:
	push bc
      ld hl,APD_BUF
      ld bc,767
      call ClearMem
      set 7,(iy+$14)
      call DisplayInfo
      res 7,(iy+$14)
      call fastCopy   ;After save game, text wouldn't display otherwise
	ld      d, 0
	ld      HL, LifeStatus
      call DTX
	ld      A, (LIFE)
	ld      H, 0
	ld      L, A
	ROM_CALL(D_HL_DECI)
 jr PauseStart
PAUSE:
 push bc
 call DisplayInfo
 ld d,$03
 ld hl,PauseT
 call DTX

PauseStart:
 call OTH_PAUSE
 call DrawScreen
 pop bc
 call ScanBk
;-------------------------------------------------------------------------------
;The game loop:
;-------------------------------------------------------------------------------
GameLoop:
 call DrawBk  ;erase ball
 ld a,c
 cp 46
 jr nz,TestKeys
 push bc
 ld a,b
 add a,8
 ld b,a
 ld c,60
 call FINDPIXEL
 ld de,BITMAPMEM
 add hl,de
 and (hl)
 pop bc
 jp z,LostLife
TestKeys:
 push bc
 ld a,$FE
 out (1),a
 in a,(1)
 ld b,a
 ld a,$BF
 out (1),a
 in a,(1)
 or $0F
 and b
 pop bc
 cp $FF
 jp z,Redraw
TestLeftKey:
 ld d,a
 ld a,(SPEED)
 ld e,a
 bit 1,d
 jr nz,TestRightKey
 ld a,b
 cp 2				; X < 2 ?
 jr c,TestRightKey		; yeah ==> cannot move left
 dec b
 ld a,e
 cp 3
 jr nz,TestRightKey		; if speed = 3, move 2 steps left
 dec b
TestRightKey:
 bit 2,d
 jr nz,TestUpKey
 ld a,b
 cp 80				; x >= 96-16 ?
 jr nc,TestUpKey
 inc b
 ld a,e
 cp 3
 jr nz,TestUpKey
 inc b
TestUpKey:
 bit 3,d
 jr nz,TestDownKey
 ld a,e
 cp 3
 jr nc,TestDownKey
 inc e
TestDownKey:
 bit 0,d
 jr nz,Test2ndKey
 ld a,e
 or a
 jr z,Test2ndKey
 dec e
Test2ndKey:
 ld a,e
 ld (SPEED),a
 bit 5,d
 jr nz,TestExitKey
 ld a,(JUMPFLAG)
 or a
 jr nz,TestExitKey
 inc a
 ld (JUMPFLAG),a
TestExitKey:		; [MODE]
 bit 6,d
 jp z,PAUSE
TestPauseKey:
 bit 7,d
 jp z,LSaveGame
Redraw:
 ld a,$FD
 out (1),a
 in a,(1)
 and $40
 jp z, GameOver

 ld a,(JUMPFLAG)
 or a
 jr z,ReallyRedraw
 dec a
 jr nz,FlyDown
 ld a,c
 cp 20
 jr z,SetFlyDown
 dec c
 jr ReallyRedraw
SetFlyDown:
 ld a,2
 ld (JUMPFLAG),a
FlyDown:
 ld a,c
 cp 46
 jr nz,FlyDown1
 xor a
 ld (JUMPFLAG),a
 jr ReallyRedraw
FlyDown1:
 inc c
ReallyRedraw:
 ld a,(SPEED)
 or a
 jr z,RedrawBall
 push bc
 ld b,a
DoScrolling:
 call DoScroll
 djnz DoScrolling
 pop bc

RedrawBall:
 call ScanBk
 ld de,WhiteBall
 call WhiteSprite
 ld de,(PLAYERPT1)
 call BlackSprite

 ld a,(ANISCROLLC)
 ld hl,SPEED
 add a,(hl)
 ld (ANISCROLLC),a
 cp 6
 jr c,GoOnMainLoop

 xor a
 ld (ANISCROLLC),a
 ld hl,ANICOUNT
 inc (hl)
 ld a,(hl)
 cp 3
 jr nz,NextFrame
 ld (hl),0
 ld hl,BlackBall1
 ld (PLAYERPT1),hl
 jr GoOnMainLoop
NextFrame:
 ld hl,(PLAYERPT1)
 ld de,32
 add hl,de
 ld (PLAYERPT1),hl


GoOnMainLoop:

 ;copy GRAPH-MEM to LCD-MEM
 push bc
 ld de,GRAPH_MEM
 ld hl,APD_BUF
 ld bc,768
 ldir 
 ei
 halt
 halt
 call fastCopy
 pop bc
  jp GameLoop

 

;---------------------------------------------------------
; WhiteSprite: draws each white element of a 24*24 sprite
;---------------------------------------------------------
WhiteSprite:
 push bc
 push de
 ld a,32	
 ld (TempByte),a
 call FINDPIXEL
 ld de,BITMAPMEM
 add hl,de
 pop de
 ld c,a
 ld b,128
WSLoop:
 ld a,(de)
 and b
 jr nz,WSLoop1
 ld a,c
 cpl
 and (hl)
 ld (hl),a
WSLoop1:
 rrc c
 jr nc,WSLoop2
 inc hl
WSLoop2:
 rrc b
 jr nc,WSLoop
WSNextLine:
 inc de
 ld a,(TempByte)
 dec a
 ld (TempByte),a
 jr z,WSEndBMP
 bit 0,a
 jr nz,WSLoop
 push de
 ld de,10
 add hl,de
 pop de
 jr WSLoop
WSEndBMP:
 pop bc
 ret

;---------------------------------------------------------
; BlackSprite: draws each black element of a 24*24 sprite
;---------------------------------------------------------
BlackSprite:
 push bc
 push de	
 ld a,32
 ld (TempByte),a
 call FINDPIXEL
 ld de,APD_BUF
 add hl,de
 pop de
 ld c,a
 ld b,128
BSLoop:
 ld a,(de)
 and b
 jr z,BSLoop1
 ld a,(hl)
 or c
 ld (hl),a
BSLoop1:
 rrc c
 jr nc,BSLoop2
 inc hl
BSLoop2:
 rrc b
 jr nc,BSLoop
BSNextLine:
 inc de
 ld a,(TempByte)
 dec a
 ld (TempByte),a
 jr z,BSEndBMP
 bit 0,a
 jr nz,BSLoop
 push de
 ld de,10
 add hl,de
 pop de
 jr BSLoop
BSEndBMP:
 pop bc
 ret



DrawScreen:

 push bc
 ld b,32
 
 ld hl, (LEVELPT)
 dec hl
 ld (LEVELPT),hl
 ld hl, LEVELY
 dec (hl)

DS_loop:
 ld hl, SCROLLY 
 inc (hl)		;increment SCROLLY
 ld d,(hl)
 ld a,d
 cp   32            ;SCROLLY = 32 ?
 jr nz, DS_same
 
 xor a			;yes, set it to 0,
 ld   d, a
 ld (hl), a
 ld hl, LEVELY
 inc (hl)
 ld hl,(LEVELPT)
 inc hl
 ld (LEVELPT),hl
 
DS_same: 
 call LoadHLScr
 
 djnz DS_loop
 pop bc
 ret

;-------------------------------------------------------------------------------
;DoScroll: handles scrolling
;-------------------------------------------------------------------------------
DoScroll:
 ld   a, (SCROLLY)   ;increment SCROLLY
 inc  a
 ld   d, a
 ld   (SCROLLY), a   ;save it
 cp   32             ;SCROLLY = 32 ?
 jr   nz, LoadHLScr
 ld   a, 0           ;yes, set it to 0,
 ld   d, 0
 ld   (SCROLLY), a
 set 7,(iy+$14)    ;Write text to GRAPH_MEM
 ld     de,$0000
 ld     HL, (SCORE)
 call   HL_DECIMAL_MENU      ;display HL menu style
 res 7,(iy+$14)     ;STop Writing text to GRAPH_MEM
 push bc            ;Save bc
 ld   de, BITMAPMEM ;Copy to APD_BUF
 ld   hl, GRAPH_MEM ;Copy from GRAPH_MEM
 ld   bc,72         ;Copy first 6 lines of GM
 ldir               ;Do it
 pop  bc            ;Restore BC

 ld a,(LEVELY)
 inc a
 cp 251
 jr nz,NoBonus
Bonus:
 ld hl,LIFE
 inc (hl)
NoBonus:
 cp 252 ;253
 jr nz,GoOnScroll
 xor a
 ld   (LEVELY), a
 ld   hl, (ORIGLVL)
 dec hl
 dec hl
 ld   (LEVELPT), hl
GoOnScroll:
 ld (LEVELY),a
 ld hl,(SCORE)
 ld d,0
 ld a,(SPEED)
 ld e,a
 add hl,de
 ld (SCORE),hl
 ld hl,(LEVELPT)
 inc hl
 ld (LEVELPT),hl

LoadHLScr:
 ld   hl, (LEVELPT)  ;get Levelinformation from LEVELPT
 inc  hl
 inc  hl
 ld   e, (hl)
 CALL DrawLine
 ld   a, (SCROLLY)
 add  a, 32
 ld   d, a
 ld   hl, (LEVELPT)
 inc hl
 ld   e, (hl)
;-------------------------------------------------------------------------------
;DrawLine: this draws a line at y-pos D, accoriding to bits in E
;-------------------------------------------------------------------------------
DrawLine:
 push   BC
 push   IX
 ld     IX, StartLine
 ld     B,  0
 ld     C,  D
 sla    C
 add    IX, BC
 add    IX, BC
;IX now contains pointer to StartLine1

 ld     L, $80
DrawLine_Loop1:
 ld     A, 47 ;63
 sub    (IX)
 ld     B, A
 ld     A, 47 ; 63
 sub    (IX+1)
 ld     C, A
 bit    4, L
 jr     z, NotMiddle
 ld     C, 47 ;63
NotMiddle:
 inc    IX

 ld     A, E
 and    L
 jr     z, WhiteLine_1
 call   BlackLine
 jr     DrawLine_Loop1_2
WhiteLine_1:
 call   WhiteLine
DrawLine_Loop1_2:
 rrc    L
 ld     A, L
 cp     $08
 jr     nz, DrawLine_Loop1

 dec    IX
 ld     B, 47 ;63
 jr     SkipMiddle
DrawLine_Loop2:
 ld     A, (IX+1)
 add    A, 47 ; 63
 ld     B, A
SkipMiddle:
 ld     A, (IX)
 add    A, 47 ;63
 ld     C, A
 dec    IX

 ld     A, E
 and    L
 jr     z, WhiteLine_2
 call   BlackLine
 jr     DrawLine_Loop2_2
WhiteLine_2:
 call   WhiteLine
DrawLine_Loop2_2:
 rrc    L
 jr     nc, DrawLine_Loop2
 pop    IX
 pop    BC
 ret

;---------------------------------------------------------------
; WhiteLine: draws a horizontal, white line from (B,D) to (C,D)
;---------------------------------------------------------------
WhiteLine:
 push hl
 push de
 ld a,c
 sub b
 push af
 ld c,d
 call FINDPIXEL
 ld de,BITMAPMEM
 add hl,de
 ld b,a
 pop af
 ld c,a
WLoop:
 ld a,b
 cpl
 and (hl)
 ld (hl),a
 dec c
 jr z,EndWhite
 rrc b
 jr nc,WLoop
 inc hl
 jr WLoop

EndWhite:
 pop  de
 pop  hl
 ret

;-------------------------------------------------------------------------------
;BlackLine: draws a horizontal, black line from (B,D) to (C,D)
;-------------------------------------------------------------------------------
BlackLine:
 push hl
 push de
 ld a,c
 sub b
 push af
 ld c,d
 call FINDPIXEL
 ld de,BITMAPMEM
 add hl,de
 ld b,a
 pop af
 ld c,a
BLoop:
 ld a,(hl)
 or b
 ld (hl),a
 dec c
 jr z,EndBlack
 rrc B
 jr nc,BLoop
 inc hl
 jr BLoop
EndBlack:
 pop de
 pop hl
 ret

;-------------------------------------------------------------------------------
;ScanBk: stores background from(B,C)-(B+24,C+24) in BKPT
;-------------------------------------------------------------------------------
ScanBk:
 push bc
 call FINDPIXEL
 ld de,BITMAPMEM
 add hl,de
 ld de,BackgroundTemp
 ld b,23
ScanLoop:
 ldi
 ldi
 ldi		;move 4 bytes from hl to de
 ldi
 push de          ;next line in vid-ram
 ld   de, 8						
 add  hl, de
 pop  de
 djnz ScanLoop
 pop  bc
 ret

;-------------------------------------------------------------------------------
;DrawBk: draws background from(B,C)-(B+24,C+24) in BKPT
;-------------------------------------------------------------------------------
DrawBk:
 push bc
 call FINDPIXEL
 ld de,BITMAPMEM
 add hl,de
 ld de,BackgroundTemp
 ex de,hl
 ld   b, 23       ;23 lines to draw
PutLoop:
 ldi
 ldi		;move 4 bytes from hl to de
 ldi
 ldi
 push hl          ;next line in vid-ram
 ld   hl, 8	
 add  hl, de
 ex   de, hl
 pop  hl
 djnz PutLoop
 pop  bc
 ret


;------------------------------------------------------------------------------
;DisplayText: displays the text in DE at position(H,L) in menustyle
;------------------------------------------------------------------------------
DisplayText:
 ld  (CURSOR_X), de
 ROM_CALL(D_ZM_STR)
 ret

DTX:
 ld e,$03
DTX2:
 ld (CURSOR_ROW),de
 ROM_CALL(D_ZT_STR)
 ret

;-------------------------------------------------------------------------------
;Some important variables:
;-------------------------------------------------------------------------------
Instr:	   .db "PJ v1.4 by A. Ess & "
SamStr:        .db "Sam Heald",0
ByStr:	   .db "By",0
EssStr:        .db "Andreas Ess",0
HiScrTxt:  .db "Hiscore: ",0
HiScrName: .db "AND",0
hiscore:   .dw 10                       ;This is standard hi-score
HiScr:     .db "NEW HIGH SCORE!",0
EnterInit: .db "ENTER YOUR INITIALS:",0
PauseT: .db "* PAUSED *",0

StartLine:
 .db 16,12,8,4	  ;#0
 .db 16,12,8,4
 .db 17,12,8,4
 .db 17,13,8,4
 .db 18,13,9,4    ;#4
 .db 18,13,9,4
 .db 19,14,9,4
 .db 19,14,9,5
 .db 20,14,10,5    ;#8
 .db 20,15,10,5
 .db 21,15,10,5
 .db 21,15,10,5
 .db 22,16,11,5    ;#12
 .db 22,16,11,5
 .db 23,16,11,6
 .db 23,17,11,6
 .db 24,17,12,6    ;#16
 .db 24,17,12,6
 .db 25,18,12,6
 .db 25,18,12,6
 .db 26,18,13,6    ;#20
 .db 26,19,13,7
 .db 27,19,13,7
 .db 27,19,13,7
 .db 28,20,14,7    ;#24
 .db 28,20,14,7
 .db 29,20,14,7
 .db 29,21,14,7
 .db 30,21,15,8    ;#28
 .db 30,21,15,8
 .db 31,22,15,8
 .db 31,22,15,8
 .db 32,22,16,8    ;#32
 .db 32,23,16,8		
 .db 33,23,16,9			 
 .db 33,23,16,9
 .db 34,24,17,9    ;#36
 .db 34,24,17,9
 .db 35,24,17,9
 .db 35,25,17,9
 .db 36,25,18,9   ;#40
 .db 36,25,18,9
 .db 37,26,18,10
 .db 37,26,18,10
 .db 38,26,19,10   ;#44
 .db 38,27,19,10
 .db 39,27,19,10
 .db 39,27,19,10
 .db 40,28,20,10    ;#48
 .db 40,28,20,11
 .db 41,28,20,11
 .db 41,29,20,11
 .db 42,29,21,11    ;#52
 .db 42,29,21,11
 .db 43,30,21,11
 .db 43,30,21,11
 .db 44,30,22,12    ;#56
 .db 44,31,22,12
 .db 45,31,22,12
 .db 45,31,22,12
 .db 46,32,23,12    ;#60
 .db 46,32,23,12
 .db 47,32,23,12
 .db 47,32,23,12


;-------------------------------------------------------------------------------
;Bitmapdata:
;-------------------------------------------------------------------------------
BackgroundTemp = TEXT_MEM2

WhiteBall:
 .db $FF,$FF,$FF,$FF,$F8,$3F,$E0,$0F,$C0,$07,$C0,$07,$80,$03,$80,$03
 .db $80,$03,$80,$03,$80,$03,$C0,$07,$C0,$07,$E0,$0F,$F8,$3F,$FF,$FF,

BlackBall1:
 .db $00,$00,$07,$C0,$18,$30,$20,$08,$40,$04,$40,$04,$80,$02,$80,$02
 .db $80,$02,$C0,$06,$A0,$0A,$54,$54,$6A,$AC,$35,$58,$1A,$B0,$07,$C0

 .db $00,$00,$07,$C0,$18,$30,$20,$08,$40,$04,$40,$04,$8A,$A2,$D5,$56
 .db $AA,$AA,$D5,$56,$8A,$A2,$40,$04,$40,$04,$20,$08,$18,$30,$07,$C0

 .db $00,$00,$07,$C0,$1A,$B0,$35,$58,$6A,$AC,$54,$54,$A0,$0A,$C0,$06
 .db $80,$02,$80,$02,$80,$02,$40,$04,$40,$04,$20,$08,$18,$30,$07,$C0

;---------------------------------------------------------------------------------
;Title Picture
;---------------------------------------------------------------------------------
TitlePicture:
    .db     16,0,8,0,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,0,0,8,0,48,16,8,4,0,0,0,0
    .db     0,0,1,131,248,0,0,2,0,0,0,0,0,0,0,127,252,16,16,3,0,0,0,0,198,2,0,62,6,48,96
    .db     35,224,0,2,3,239,128,0,24,131,60,224,15,48,0,0,7,109,224,128,26,113,255,192,6,28,0,8,14,120,112
    .db     72,52,249,195,192,6,159,144,0,28,56,24,121,180,252,1,193,6,220,0,16,56,48,184,61,241,252,0,228,119,48
    .db     0,48,112,17,176,63,241,252,60,120,255,240,112,96,232,139,112,55,113,252,126,120,223,225,252,224,118,56,102,119,112
    .db     249,126,113,143,243,15,192,57,246,111,99,114,3,126,97,143,158,3,192,31,179,253,226,49,143,60,225,143,30,0,224
    .db     31,51,249,224,17,254,1,227,143,31,0,112,63,51,240,192,16,248,135,247,7,31,32,48,63,51,248,64,18,192,127
    .db     255,7,31,112,56,103,51,248,64,16,96,63,255,39,31,60,24,195,49,248,64,9,96,31,222,115,31,63,29,131,24
    .db     249,80,137,96,31,142,115,31,63,31,3,88,48,77,1,184,15,158,115,95,63,31,143,76,1,71,64,190,71,156,3
    .db     30,63,31,254,44,2,71,164,219,163,28,49,94,191,63,252,108,134,199,184,216,177,28,9,92,62,56,240,222,124,135
    .db     180,220,216,184,113,156,190,58,0,63,57,142,205,204,209,62,120,153,188,54,3,243,195,204,124,60,230,65,120,250,121
    .db     19,31,193,254,252,55,248,111,58,120,253,250,0,255,131,252,124,7,192,127,196,124,127,187,7,253,131,28,24,3,0
    .db     121,255,236,118,25,175,224,196,12,24,2,0,48,127,207,224,28,31,0,64,4,8,6,0,0,31,199,192,63,252,0
    .db     0,2,8,4,0,0,3,128,0,63,240,0,32,0,0,4,0,0,1,128,0,97,192,0,0,0,136,0,0,0,0
    .db     192,1,0,128,0,0,0,0,8,0,0,0,64,0,0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0





;-----------------------------------------------------------------------------
;HL_DECIMAL_MENU(Magnus Hagander):Writes the in HL stored word in menu font
;-----------------------------------------------------------------------------
HL_DECIMAL_MENU:
 ld (CURSOR_X),de
 push bc
 ld   de, StringPlace+4
 ld   b,  5
ConvLoop:
 call UNPACK_HL
 add  a,'0'
 ld   (de), a
 dec  de
 djnz ConvLoop
 ld   hl, StringPlace
 ROM_CALL(D_ZM_STR)
 pop bc
 ret




;-------------------------------------------------------------------------------
;FINDPIXEL Routine, by Patrick Davidson [From Zkart3d 82]
;-------------------------------------------------------------------------------
FINDPIXEL_DATA:
         .db      128,64,32,16,8,4,2,1
FINDPIXEL:
         push     bc
         ld       a,b
         and      7
         ld       hl,FINDPIXEL_DATA
         ld       e,a
         ld       d,0
         add      hl,de
         ld       e,(hl)            ; E = pixel mask

	
         ld a,c	

         add      a,a
         add      a,c               ; A = 3 * Y
         ld       l,a
         ld       h,0
         add      hl,hl
         add      hl,hl             ; HL = 12 * Y
         ld       a,b
         ld       b,0
         rrca
         rrca
         rrca
         and      15
         ld       c,a
         add      hl,bc
         ld       a,e               ; A = pixel mask
         pop      bc
         ret


LifeStatus: .db "Lives Left:",0



DisplayInfo:
 push bc
 ROM_CALL(CLEARLCD)
 call _GRBUFCLR
 ld hl,GRAPH_MEM
 call Invert
 ld hl,GRAPH_MEM+672
 call Invert2
 ld de,$0003
 set 3,(iy+$05)
 ld hl,Instr
 call DisplayText
 ld de,$3909
 ld hl,HiScrTxt
 call DisplayText

 ld de,$392D
 call DisplayText

 ld de,$3942
 ld hl,hiscore
 call LD_HL_MHL
 call HL_DECIMAL_MENU
 res 3,(iy+$05)
 pop bc
 ret


LoadLevel:
 call DisplayInfo
 ld ix,Levels
 xor a
 ld (LvlNo),a
 ld hl,$FE6E		; 83 : start of program VAT ( reverse )
SearchLoop:
 ld de, ($8D12)        ; end of progtable - 1
 inc de
 or a
 sbc hl, de
 jr c, SearchDone
 add hl,de
 ld a,(hl)
 dec hl
 ld e,(hl)
 dec hl
 ld d,(hl)		; DE = ptr to variable data
 push hl
 sub 5			; type = 5/6 ?
 jr z, ProgFound
 dec a
 jr z, ProgFound	; no
 pop hl
 ld b,4
 jr SkipName


ProgFound:
 ex de,hl	; HL -> variable data
 inc hl		; (skip length)
 inc hl
 ld a, (hl)	; A = byte at HL
 cp 3		; Check first header byte
 jr nz,ContinueSearch
 inc hl
 ld a, (hl)
 or a		; Check second header byte
 jr nz,ContinueSearch
built_in:
 pop hl
 dec hl
 ld b,(hl) 	; B = var name len
 ld (ix),b
 push ix
CopyName:
 inc ix
 dec hl
 ld a,(hl)
 ld (ix),a
 djnz CopyName
 dec hl
 pop ix
 ld de,9
 add ix,de
 ld a,(LvlNo)
 inc a
 ld (LvlNo),a
 jr SearchLoop

ContinueSearch:
 pop hl
 dec hl                ; skip type & ptr
 ld b, (hl)            ; retrieve name length
 inc b                 ; inc so also skip namelen byte
SkipName:
 dec hl
 djnz SkipName
 jr SearchLoop

SearchDone:
 xor a
 ld (SetD+1),a
 ld a,(LvlNo)
 or a
 jr nz,tt
NoLevels:
 ld de, $0801
 ld hl, NoLevel
 call DisplayText
 call fastCopy
 call OTH_PAUSE
 res 7,(iy+$14)
 scf
 ret

tt:
 xor a
 ld (Top),a




ShowLevels:
 ld hl,$0801
 ld (CURSOR_X),hl
 ld hl,GRAPH_MEM+(7*12)		;+112
 ld bc,	50*12-1			;799
 call ClearMem
 ld hl,(Top)
 ld h,9
 ROM_CALL($1F18-$1A)  
 ld de,Levels
 add hl,de
 ld b,8
 ld a,(LvlNo)
 cp 8
 jr nc,ShowLevel
 ld b,a
ShowLevel:
 push hl
 push bc
 ld b,(hl)
 inc hl
 ROM_CALL(D_LM_STR)
 pop bc
 pop hl
 ld de,9
 add hl,de
 ld a,(CURSOR_Y)
 add a,6
 ld (CURSOR_Y),a
 ld a,1
 ld (CURSOR_X),a
 djnz ShowLevel
 ld hl,Top
 ld a,(LvlNo)
 sub (hl)
 cp 8
 jr c,SetMax
 ld a,8
SetMax:
 ld e,a
 dec e
SetD:
 ld d,0
 call ShowBar
CL_KeyLoop:
 push de
 call GET_KEY
 pop de
 cp G_DOWN
 jr z,ScrollDown
 cp G_UP
 jr z,ScrollUp
 cp G_ENTER
 jr z,AcceptLevel
 cp G_CLEAR
 jr nz,CL_KeyLoop
 scf
 res 7,(iy+$14)
 ret
ScrollDown:
 call ShowBar
 ld a,d
 cp e
 jr z,ScrollL_Down
 inc d
 jr ShowIt
ScrollL_Down:
 ld (SetD+1),a
 ld a,(LvlNo)
 sub 8
 jr c,ShowIt
 ld hl,Top
 cp (hl)
 jr z,ShowIt
 inc (hl)
 jr Sh
ScrollL_Up:
 ld (SetD+1),a
 ld a,(Top)
 or a
 jr z, ShowIt
 dec a
 ld (Top),a
Sh:
 jp ShowLevels
ScrollUp:
 call ShowBar
 ld a,d
 or a
 jr z,ScrollL_Up
 dec d
ShowIt:
 call ShowBar
 jr CL_KeyLoop
AcceptLevel:
 ld a,(Top)
 add a,d
 ld l,a
 ld h,9
 ROM_CALL($1F18-$1A)
 ld de,Levels
 add hl,de
 push hl
 ld de,LEVELNAME

_mov9b = $1A15 ;1A93
 call _mov9b    ;** From the 86


 pop hl

ReadLevel:     ; Read level hl
; dec hl  ; First byte at _OP1 should not be used
; call _mov10toop1   ;** from the 86
 rst 20h
 ld a,5
 ld ($8028),a		; set type = program
 ROM_CALL(213Eh-1Ah)  ;** from the 86
 ret c
 ld hl,4
 add hl, de		; skip length of program + header bytes
 ret			; return hl = pointer to the program.

ShowBar:
 push de
 ld l,d
 ld h,72
 ROM_CALL($1F18-$1A)  
 ld de,GRAPH_MEM+(8*12)
 add hl,de
 call Invert
 pop de
 ret

Invert2:
 ld bc,96
 jr InvLoop

Invert:
 ld bc,84
InvLoop:
 ld a,(hl)
 cpl
 ld (hl),a
 inc hl
 dec bc
 ld a,b
 or c
 jr nz,InvLoop
fastCopy:
	di			; 4
	ld	a,$80		; 7
	out	($10),a		; 11
	ld	hl,GRAPH_MEM-12-(-(12*64)+1)	; 10
	ld	a,$20		; 7
	ld	c,a		; 4	; 43
fastCopyAgain:
	ld	b,64		; 7
	inc	c		; 4
	ld	de,-(12*64)+1	; 10
	out	($10),a		; 11
	add	hl,de		; 11
	ld	de,11		; 10
fastCopyLoop:
	add	hl,de		; 11
	inc	hl		; 6
	ret	c		; 5	; do nothing instruction (was nop (4 clocks))
	ld	a,(hl)		; 7
	out	($11),a		; 11
	djnz	fastCopyLoop	; 13/8	; 3392
	ld	a,c		; 4
	cp	$2B+1		; 7
	jr	nz,fastCopyAgain; 12/7
	ret	; 11	; 18

GameOver:
 call GET_KEY ;Clear last key press
 ld  hl, LevelSaved
 xor a
 ld (hl),a

hiscoreHand:
 ld hl, hiscore
 call LD_HL_MHL
 ld de,(SCORE)
 call CP_HL_DE
 jp nc,ExitGame
 ld hl,hiscore
 ld (hl),e
 inc hl
 ld (hl),d
HiWrite:
 ROM_CALL(CLEARLCD)
 ld de,$0100
 ld hl,HiScr
 call DTX2
 ld de,$0A0F
 call DisplayText
 ld hl, $0603
 ld (CURSOR_ROW),hl
 ld hl,HiScrName
         push     hl
         ld       b,3
space_loop:
         ld       (hl), ' '
         inc      hl
         djnz     space_loop
         pop      ix

enter_name_loop:
	   call     GET_KEY
         or       A
         jr       z,enter_name_loop
         cp       $38
         jr       z,backup
         SUB      9
         jp      z,ExitGame
         dec      a
         jp       m,enter_name_loop
         ld       hl,chartable
         ld       e,a
         ld       d,0
         add      hl,de
         ld       a,3
         cp       b
         jr       z,enter_name_loop
         ld       a,(hl)
         or       a
         jr       z,enter_name_loop
         ld       (ix),a
	   ROM_CALL(TX_CHARPUT)
         inc      b
         inc      ix
         jr       enter_name_loop
backup:  xor      a
         cp       b
         jr       z,enter_name_loop
         dec      b
         dec      ix
         ld       a,' '

         ld       (ix),a
         ld       hl,CURSOR_COL
         dec      (hl)
	   ROM_CALL(TX_CHARPUT)
         dec      (hl)
         jr       enter_name_loop


chartable:
         .db      ":WRMH."
         .DB      "..0VQLG!..ZUPKFC"
         .DB      "..YTOJEBX.>SNIDA"
         .DB      ".12345.."

NoLevel:    .db "No levels found!",0

LEVELNAME:	.db 0,0,0,0,0,0,0,0,0		; name of the level (must be save when game paused)

LevelSaved:	.db 0  ;standard: no level saved

LEVELY:		.db 0	;position in level
SCROLLY:	.db 0	;to_level_data	;position in level-y
LEVELPT:	.dw 0	;pointer 
BKPT:		.dw 0	;pointer to background
PLAYERPT:	.dw 0	;pointer to player
JUMPFLAG:	.db 0	;0 = no jump; 1 = up; 2 = down
PLAYERPT1:	.dw 0	;pointer to black-player image
ANICOUNT:	.db 0	;animation counter
SPEED:		.db 0	;speed of ball
LIFE:		.db 0	;lifes of player
SCORE:		.dw 0	;contains player's score
BALLpos:	.dw 0	;only needed for savegame

_GRBUFCLR:
 ld hl,GRAPH_MEM
 ld (hl),0
 ld de,GRAPH_MEM+1
 ld bc,767
 ldir
 ret

.end


